# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - Export as PDF action

    @copyright: 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin.wikiutil import escape
from MoinSupport import ActionSupport, escattr, getFormatterClass, formatText, get_send_headers
from os.path import join
import subprocess

__version__ = "0.1"

Dependencies = ['page']

# Configuration settings.

XSLT_PROCESSOR              = "/usr/bin/xsltproc"
FO_PROCESSOR                = "/usr/bin/fop"
DOCBOOK_STYLESHEET_BASE     = "/usr/share/xml/docbook/stylesheet"

# Tool settings.

DOCBOOK_TO_FO_STYLESHEET    = "docbook-xsl/fo/docbook.xsl"

# NOTE: From docbook-xsl/fo/param.xsl.

paper_sizes = [
    "A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7", "A8", "A9", "A10",
    "B0", "B1", "B2", "B3", "B4", "B5", "B6", "B7", "B8", "B9", "B10",
    "C0", "C1", "C2", "C3", "C4", "C5", "C6", "C7", "C8", "C9", "C10",
    "A4landscape", "USletter", "USlandscape", "4A0", "2A0",
    ]

paper_size_labels = {
    "A4landscape" : "A4 landscape",
    "USletter" : "US letter",
    "USlandscape" : "US landscape",
    "4A0" : "Quadruple A0",
    "2A0" : "Double A0"
    }

class ExportPDF(ActionBase, ActionSupport):

    "Export the current page as PDF."

    def get_form_html(self, buttons_html):

        "Return the action's form incorporating the 'buttons_html'."

        _ = self._
        request = self.request
        form = self.get_form()

        paper_size = form.get("paper-size", ["A4"])[0]

        paper_size_options = []

        for size in paper_sizes:
            paper_size_options.append('<option value="%s" %s>%s</option>' % (
                escattr(size), self._get_selected(size, paper_size),
                escape(_(paper_size_labels.get(size) or size))
                ))

        d = {
            "paper_size_label"      : escape(_("Paper size")),
            "paper_size_options"    : u"".join(paper_size_options),
            "buttons_html"          : buttons_html,
            }

        return u"""\
<table>
    <tr>
        <td class="label"><label>%(paper_size_label)s</label></td>
        <td><select name="paper-size">%(paper_size_options)s</select></td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">%(buttons_html)s</td>
    </tr>
</table>
""" % d

    def do_action(self):

        "Attempt to post a comment."

        _ = self._
        request = self.request
        form = self.get_form()
        page = self.page

        paper_size = form.get("paper-size", [""])[0]

        if not paper_size in paper_sizes:
            return 0, _("A paper size must be chosen.")

        # Get the page in DocBook format.

        fmt = getFormatterClass(request, "text_docbook")(request)
        fmt.setPage(page)

        # The DocBook formatter needs to pretend a full document is being made.

        page_as_docbook = []
        append = page_as_docbook.append

        append(fmt.startDocument(page.page_name))
        append(fmt.startContent())
        append(formatText(page.get_raw_body(), request, fmt, inhibit_p=False).encode("utf-8"))
        append(fmt.endContent())
        append(fmt.endDocument())

        # Send the DocBook XML to the XSLT processor.

        p1 = subprocess.Popen([
            XSLT_PROCESSOR,
            "-stringparam", "fop1.extensions", "1",
            "--stringparam", "paper.type", paper_size,
            join(DOCBOOK_STYLESHEET_BASE, DOCBOOK_TO_FO_STYLESHEET),
            "-"
            ],
            shell=False,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)

        p1.stdin.write("".join(page_as_docbook))
        p1.stdin.close()

        # Pipe the XML-FO output to the FO processor.

        p2 = subprocess.Popen([
            FO_PROCESSOR,
            "-fo", "-",
            "-pdf", "-",
            ],
            shell=False,
            stdin=p1.stdout,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)

        out, err = p2.communicate()

        retcode = p1.wait()

        if retcode != 0:
            return 0, err

        retcode = p2.wait()

        if retcode != 0:
            return 0, err

        send_headers = get_send_headers(request)
        headers = ["Content-Type: application/pdf"]
        send_headers(headers)
        request.write(out)

        return 1, None

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since a resource has already been
        produced.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

# Action invocation function.

def execute(pagename, request):
    ExportPDF(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
